/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/***************************************************************************
 *            gtkvumeter.c
 *
 *  Fri Jan 10 20:06:23 2003
 *  Copyright  2003  Todd Goyen
 *  wettoad@knighthoodofbuh.org
 *  Modified by Iain Holmes - Copyright (C) 2004-2005
 ****************************************************************************/

#include <math.h>
#include <gtk/gtk.h>

#include <marlin/marlin-types.h>

#include "gtkvumeter.h"

#define MIN_HORIZONTAL_VUMETER_WIDTH   400
#define HORIZONTAL_VUMETER_HEIGHT  20
#define VERTICAL_VUMETER_WIDTH     20
#define MIN_VERTICAL_VUMETER_HEIGHT    200

#define MAX_DB 0.0
#define MIN_DB -96.0

static void gtk_vumeter_init (GtkVUMeter *vumeter);
static void gtk_vumeter_class_init (GtkVUMeterClass *class);
static void gtk_vumeter_finalize (GObject *object);
static void gtk_vumeter_realize (GtkWidget *widget);
static void gtk_vumeter_size_request (GtkWidget *widget, GtkRequisition *requisition);
static void gtk_vumeter_size_allocate (GtkWidget *widget, GtkAllocation *allocation);
static gint gtk_vumeter_expose (GtkWidget *widget, GdkEventExpose *event);
static void gtk_vumeter_free_colors (GtkVUMeter *vumeter);
static void gtk_vumeter_setup_colors (GtkVUMeter *vumeter);
static gint gtk_vumeter_sound_level_to_draw_level (GtkVUMeter *vumeter, double level);

static GtkWidgetClass *parent_class = NULL;

enum {
	PROP_0,
	PROP_SCALE
};

GType 
gtk_vumeter_get_type (void)
{
	static GType vumeter_type = 0;
	
	if (!vumeter_type) {
		static const GTypeInfo vumeter_info = {
			sizeof (GtkVUMeterClass),
			NULL, NULL,
			(GClassInitFunc) gtk_vumeter_class_init,
			NULL, NULL, sizeof (GtkVUMeter), 0,
			(GInstanceInitFunc) gtk_vumeter_init,
		};

		vumeter_type = g_type_register_static (GTK_TYPE_WIDGET, 
						       "GtkVUMeter", 
						       &vumeter_info, 0);
	}
	
	return vumeter_type;
}

/* Orientation should be a property */
GtkWidget* 
gtk_vumeter_new (GtkVUMeterOrientation orientation)
{
	GtkVUMeter *vumeter;

	vumeter = gtk_type_new (GTK_TYPE_VUMETER);
	vumeter->orientation = orientation;
	vumeter->rms_level = MIN_DB;
	vumeter->peak_level = MIN_DB;

	return GTK_WIDGET (vumeter);
}

static void 
gtk_vumeter_init (GtkVUMeter *vumeter)
{
	vumeter->peaks_falloff = GTK_VUMETER_PEAKS_FALLOFF_MEDIUM;
	
	vumeter->scale = MARLIN_SCALE_LOG;
}

static void
set_property (GObject *object,
	      guint prop_id,
	      const GValue *value,
	      GParamSpec *pspec)
{
	GtkVUMeter *vu = GTK_VUMETER (object);
	GtkWidget *widget = GTK_WIDGET (object);

	switch (prop_id) {
	case PROP_SCALE:
		vu->scale = g_value_get_enum (value);

		if (GTK_WIDGET_DRAWABLE (widget)) {
			GdkRectangle rect;

			rect.x = 0;
			rect.y = 0;
			rect.width = widget->allocation.width;
			rect.height = widget->allocation.height;

			gdk_window_invalidate_rect (widget->window, &rect, FALSE);
		}
		break;

	default:
		break;
	}
}

static void
get_property (GObject *object,
	      guint prop_id,
	      GValue *value,
	      GParamSpec *pspec)
{
}

static void 
gtk_vumeter_class_init (GtkVUMeterClass *class)
{
	GObjectClass *object_class;
	GtkWidgetClass *widget_class;
	
	object_class = (GObjectClass*) class;
	widget_class = (GtkWidgetClass*) class;
	parent_class = gtk_type_class(gtk_widget_get_type());
	
	object_class->finalize = gtk_vumeter_finalize;
	object_class->set_property = set_property;
	object_class->get_property = get_property;
	
	widget_class->realize = gtk_vumeter_realize;
	widget_class->expose_event = gtk_vumeter_expose;
	widget_class->size_request = gtk_vumeter_size_request;
	widget_class->size_allocate = gtk_vumeter_size_allocate;    

	g_object_class_install_property (object_class,
					 PROP_SCALE,
					 g_param_spec_enum ("scale",
							    "", "",
							    MARLIN_TYPE_SCALE,
							    MARLIN_SCALE_LINEAR,
							    G_PARAM_READWRITE));
}

static void 
gtk_vumeter_finalize (GObject *object)
{
	GtkVUMeter *vumeter = GTK_VUMETER (object);
	
	gtk_vumeter_free_colors (vumeter);
	
	G_OBJECT_CLASS (parent_class)->finalize (object);
}

static void 
gtk_vumeter_realize (GtkWidget *widget)
{
	GtkVUMeter *vumeter;
	GdkWindowAttr attributes;
	int attributes_mask;
	int i;
	
	g_return_if_fail (GTK_IS_VUMETER (widget));
	
	GTK_WIDGET_SET_FLAGS (widget, GTK_REALIZED);
	vumeter = GTK_VUMETER (widget);
	
	attributes.x = widget->allocation.x;
	attributes.y = widget->allocation.y;
	attributes.width = widget->allocation.width;
	attributes.height = widget->allocation.height;
	attributes.wclass = GDK_INPUT_OUTPUT;
	attributes.window_type = GDK_WINDOW_CHILD;
	attributes.event_mask = gtk_widget_get_events (widget) | GDK_EXPOSURE_MASK;
	attributes.visual = gtk_widget_get_visual (widget);
	attributes.colormap = gtk_widget_get_colormap (widget);
	attributes_mask = GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL | GDK_WA_COLORMAP;
	widget->window = gdk_window_new (widget->parent->window, &attributes, attributes_mask);
	
	widget->style = gtk_style_attach (widget->style, widget->window);
	
	gdk_window_set_user_data (widget->window, widget);
	gtk_style_set_background (widget->style, widget->window,  GTK_STATE_NORMAL);
	
	/* colors */
	vumeter->colormap = gdk_colormap_get_system ();
	gtk_vumeter_setup_colors (vumeter);
}

static void 
gtk_vumeter_size_request (GtkWidget *widget, 
			  GtkRequisition *requisition)
{
	GtkVUMeter *vumeter;
	
	g_return_if_fail (GTK_IS_VUMETER (widget));
	g_return_if_fail (requisition != NULL);
	
	vumeter = GTK_VUMETER (widget);
	
	switch (vumeter->orientation) {
	case GTK_VUMETER_VERTICAL:
		requisition->width = VERTICAL_VUMETER_WIDTH;
		requisition->height = -1;
		break;

	case GTK_VUMETER_HORIZONTAL:
		requisition->width = -1;
 		requisition->height = HORIZONTAL_VUMETER_HEIGHT;
		break;
	}
}

static void 
gtk_vumeter_size_allocate (GtkWidget *widget,
			   GtkAllocation *allocation)
{
	GtkVUMeter *vumeter;
	int red, green;
	int f_step;
	int b_step;
	
	widget->allocation = *allocation;
	vumeter = GTK_VUMETER (widget);
	
	if (GTK_WIDGET_REALIZED (widget) == FALSE) {
		return;
	}

	switch (vumeter->orientation) {
	case GTK_VUMETER_VERTICAL:
		gdk_window_move_resize (widget->window, 
					allocation->x, allocation->y,
					VERTICAL_VUMETER_WIDTH, allocation->height);
		break;
		
	case GTK_VUMETER_HORIZONTAL:
		gdk_window_move_resize (widget->window, 
					allocation->x, allocation->y,
					allocation->width, HORIZONTAL_VUMETER_HEIGHT);
		break;
		
	default:
		break;
	}
	
	gtk_vumeter_setup_colors (vumeter);
}

static gint 
gtk_vumeter_expose (GtkWidget *widget, 
		    GdkEventExpose *event)
{
	GtkVUMeter *vumeter;
	int index, rms_level, peak_level;
	int width, height;
	GdkGC *gc;

	if (event->count > 0) {
		return FALSE;
	}
	
	vumeter = GTK_VUMETER (widget);
	rms_level = gtk_vumeter_sound_level_to_draw_level (vumeter, 
							   vumeter->rms_level);
	peak_level = gtk_vumeter_sound_level_to_draw_level (vumeter, 
							    vumeter->peak_level);

	gc = gdk_gc_new (widget->window);

	switch (vumeter->orientation) {
	case GTK_VUMETER_VERTICAL:
		width = widget->allocation.width - 2;
		height = widget->allocation.height;
		
		/* draw border */
		gtk_paint_box (widget->style, widget->window, 
			       GTK_STATE_NORMAL, GTK_SHADOW_IN, 
			       NULL, widget, "trough", 0, 0, 
			       widget->allocation.width, height);
		
		gdk_gc_copy (gc, widget->style->bg_gc[GTK_STATE_NORMAL]);

		for (index = peak_level; index < rms_level; index++) {
			gdk_gc_set_foreground (gc, &vumeter->b_colors[index]);
			gdk_draw_line (widget->window, gc, 
				       1, index + 1, 
				       width, index + 1);   
		}
		
		gdk_gc_copy (gc, widget->style->fg_gc[GTK_STATE_NORMAL]);
		for (index = widget->allocation.height - 1; index > rms_level; index--) {
			gdk_gc_set_foreground (gc, &vumeter->f_colors[index]);
			gdk_draw_line (widget->window, gc, 
				       1, index + 1, 
				       width, index + 1);
		}

		break;

	case GTK_VUMETER_HORIZONTAL:
		width = widget->allocation.width;
		height = widget->allocation.height - 2;
		
		/* draw border */
		gtk_paint_box (widget->style, widget->window, 
			       GTK_STATE_NORMAL, GTK_SHADOW_IN, 
			       NULL, widget, "trough", 0, 0, 
			       width, widget->allocation.height);

		gdk_gc_copy (gc, widget->style->bg_gc[GTK_STATE_NORMAL]);
		for (index = rms_level; index < peak_level; index++) {
			gdk_gc_set_foreground (gc, &vumeter->b_colors[index]);
			gdk_draw_line (widget->window, gc, 
				       width - index - 1, 1, 
				       width - index - 1, height);   
		}

		gdk_gc_copy (gc, widget->style->fg_gc[GTK_STATE_NORMAL]);
		for (index = peak_level; index < width - 2; index++) {
			gdk_gc_set_foreground (gc, &vumeter->f_colors[index]);
			gdk_draw_line (widget->window, gc,
				       width - index - 1, 1, 
				       width - index - 1, height);            
		}

		break;

	default:
		break;
	}
	
	g_object_unref (G_OBJECT (gc));
}

static void 
gtk_vumeter_free_colors (GtkVUMeter *vumeter)
{
	int index;
	
	if (vumeter->f_colors) {
		gdk_colormap_free_colors (vumeter->colormap, vumeter->f_colors,
					  vumeter->colors);
		g_free (vumeter->f_colors);
		vumeter->f_colors = NULL;
	}

	if (vumeter->b_colors) {
		gdk_colormap_free_colors (vumeter->colormap, vumeter->b_colors,
					  vumeter->colors);
		g_free (vumeter->b_colors);
		vumeter->b_colors = NULL;
	}
}

static void 
gtk_vumeter_setup_colors (GtkVUMeter *vumeter)
{
	int index, colours;
	int f_step, b_step;
	int first, second;
	int max = 0, min = 0, log_max = 0;
	
	g_return_if_fail (vumeter->colormap != NULL);
	
	if (vumeter->orientation == GTK_VUMETER_VERTICAL) {
		colours = MAX (GTK_WIDGET (vumeter)->allocation.height - 2, 0);
	} else {
		colours = MAX (GTK_WIDGET (vumeter)->allocation.width - 2, 0);
	}
	
	if (colours == vumeter->colors) {
		return;
	}
	
	gtk_vumeter_free_colors (vumeter);
	
	vumeter->colors = colours;
	if (vumeter->colors == 0)
		return;
	
	vumeter->f_colors = g_malloc (vumeter->colors * sizeof (GdkColor));
	vumeter->b_colors = g_malloc (vumeter->colors * sizeof (GdkColor));    
	
	if (vumeter->scale == MARLIN_SCALE_LINEAR) {
		first = vumeter->colors / 2;
		second = vumeter->colors;      
	} else {
		log_max = -20 * log10 (1.0 / 20.0);

		first = (int) ((double) vumeter->colors * (6.0 / log_max));
		second = (int) ((double) vumeter->colors * (18.0 / log_max));
	}

	vumeter->f_colors[0].red = 65535;
	vumeter->f_colors[0].green = 0;
	vumeter->f_colors[0].blue = 0;
	
	vumeter->b_colors[0].red = 49151;
	vumeter->b_colors[0].green = 0;
	vumeter->b_colors[0].blue = 0;
	
	/* Allocate from Red to Yellow */
	f_step = 65535 / (first - 1);
	b_step = 49151 / (first - 1);

	for (index = 1; index < first; index++) {
		vumeter->f_colors[index].red = 65535;
		vumeter->f_colors[index].green = vumeter->f_colors[index - 1].green + f_step;
		vumeter->f_colors[index].blue = 0;

		vumeter->b_colors[index].red = 49151;
		vumeter->b_colors[index].green = vumeter->b_colors[index - 1].green + b_step;
		vumeter->b_colors[index].blue = 0;
	}

	/* Allocate from Yellow to Green */    
	f_step = 65535 / (second - first);
	b_step = 49151 / (second - first);        

	for (index = first; index < second; index++) {
		vumeter->f_colors[index].red = vumeter->f_colors[index - 1].red - f_step;
		vumeter->f_colors[index].green = vumeter->f_colors[index - 1].green;
		vumeter->f_colors[index].blue = 0;

		vumeter->b_colors[index].red = vumeter->b_colors[index - 1].red - b_step;
		vumeter->b_colors[index].green = vumeter->b_colors[index - 1].green;
		vumeter->b_colors[index].blue = 0;     
	}

	if (vumeter->scale == MARLIN_SCALE_LOG) {
		/* Allocate from Green to Dark Green */
		f_step = 32767 / (vumeter->colors - second);
		b_step = 32767 / (vumeter->colors - second);     

		for (index = second; index < vumeter->colors; index++) {
			vumeter->f_colors[index].red = 0;
			vumeter->f_colors[index].green = vumeter->f_colors[index - 1].green - f_step;
			vumeter->f_colors[index].blue = 0;

			vumeter->b_colors[index].red = 0;
			vumeter->b_colors[index].green = vumeter->b_colors[index - 1].green - b_step;
			vumeter->b_colors[index].blue = 0;      
		}
	}

	for (index = 0; index < vumeter->colors; index++) {
		gdk_colormap_alloc_color (vumeter->colormap, 
					  &vumeter->f_colors[index], 
					  FALSE, TRUE);

		gdk_colormap_alloc_color (vumeter->colormap, 
					  &vumeter->b_colors[index], 
					  FALSE, TRUE);
	}
}

static int 
gtk_vumeter_sound_level_to_draw_level (GtkVUMeter *vumeter, 
				       double level)
{
	double draw_level;
	double height;
	double log_level, log_max;
	
	height = (double)GTK_WIDGET (vumeter)->allocation.height - 2;
	
	if (vumeter->scale == MARLIN_SCALE_LINEAR) {
	} else {
		log_level = level / -96.0;
		draw_level = log_level * height;
	}
	
	return (int) draw_level;
}

#if 0
void 
gtk_vumeter_set_min_max (GtkVUMeter *vumeter, 
			 int min, 
			 int max)
{
	g_return_if_fail (vumeter != NULL);
	g_return_if_fail (GTK_IS_VUMETER (vumeter));
	
	vumeter->max = max;
	vumeter->min = min;
	
	if (vumeter->max == vumeter->min) {
		vumeter->max++;
	}
	
	vumeter->rms_level = CLAMP (vumeter->rms_level, vumeter->min, 
				    vumeter->max);
	vumeter->peak_level = CLAMP (vumeter->peak_level, vumeter->min, 
				     vumeter->max);

	gtk_widget_queue_draw (GTK_WIDGET (vumeter)); 
}
#endif

void 
gtk_vumeter_set_levels (GtkVUMeter *vumeter, 
			double rms, 
			double peak)
{
	g_return_if_fail (vumeter != NULL);
	g_return_if_fail (GTK_IS_VUMETER (vumeter));
	
	vumeter->rms_level = CLAMP (rms, MIN_DB, MAX_DB);
	vumeter->peak_level = CLAMP (peak, MIN_DB, MAX_DB);
	
	gtk_widget_queue_draw (GTK_WIDGET (vumeter));    
}

/* Should be set as a property */
void 
gtk_vumeter_set_peaks_falloff (GtkVUMeter *vumeter, 
			       GtkVUMeterFallOff falloff)
{
	g_return_if_fail (GTK_IS_VUMETER (vumeter));

	vumeter->peaks_falloff = falloff;
}
